==> redux-0.2.1.tgz: package/lib/createDispatcher.js <==
'use strict';

var _Object$defineProperty = require('babel-runtime/core-js/object/define-property')['default'];

var _Object$keys = require('babel-runtime/core-js/object/keys')['default'];

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default')['default'];

_Object$defineProperty(exports, '__esModule', {
  value: true
});

exports['default'] = createDispatcher;

var _lodashObjectMapValues = require('lodash/object/mapValues');

var _lodashObjectMapValues2 = _interopRequireDefault(_lodashObjectMapValues);

// An action dispatched to init store state
var BOOTSTRAP_STORE = {
  type: 'BOOTSTRAP_STORE'
};

function createDispatcher() {
  var observers = {};
  var stores = {};
  var actionCreators = {};
  var currentState = {};
  var currentTransaction = null;
  var committedState = {};

  // To compute the next state, combine the next states of every store
  function computeNextState(state, action) {
    return (0, _lodashObjectMapValues2['default'])(stores, function (store, key) {
      return store(state[key], action);
    });
  }

  // Notify observers about the changed stores
  function emitChange(changedKeys) {
    if (!changedKeys.length) {
      return;
    }

    // Gather the affected observers
    var notifyObservers = [];
    changedKeys.forEach(function (key) {
      observers[key].forEach(function (o) {
        if (notifyObservers.indexOf(o) === -1) {
          notifyObservers.push(o);
        }
      });
    });

    // Emit change
    notifyObservers.forEach(function (o) {
      return o();
    });
  }

  // Update state and emit change if needed
  function updateState(nextState) {
    // Swap the state
    var previousState = currentState;
    currentState = nextState;

    // Notify the observers
    var changedKeys = _Object$keys(currentState).filter(function (key) {
      return currentState[key] !== previousState[key];
    });
    emitChange(changedKeys);
  }

  // Reassign the current state on each dispatch
  function dispatch(action) {
    if (typeof action.type !== 'string') {
      throw new Error('Action type must be a string.');
    }

    var nextState = computeNextState(currentState, action);
    updateState(nextState);
  }

  // Provide subscription and unsubscription
  function observeStores(observedKeys, onChange) {
    // Emit the state update
    function handleChange() {
      onChange(currentState);
    }

    // Synchronously emit the initial value
    handleChange();

    // Register the observer for each relevant key
    observedKeys.forEach(function (key) {
      return observers[key].push(handleChange);
    });

    // Let it unregister when the time comes
    return function () {
      observedKeys.forEach(function (key) {
        var index = observers[key].indexOf(handleChange);
        observers[key].splice(index, 1);
      });
    };
  }

  // Dispatch in the context of current transaction
  function dispatchInTransaction(action) {
    if (currentTransaction) {
      currentTransaction.push(action);
    }
    dispatch(action);
  }

  // Bind action creator to the dispatcher
  function wrapActionCreator(actionCreator) {
    return function dispatchAction() {
      for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }

      var action = actionCreator.apply(undefined, args);
      if (typeof action === 'function') {
        // Async action creator
        action(dispatchInTransaction);
      } else {
        // Sync action creator
        dispatchInTransaction(action);
      }
    };
  }

  // Provide dispatching
  function getActions() {
    return actionCreators;
  }

  // Provide a way to receive new stores and actions
  function receive(nextStores, nextActionCreators) {
    stores = nextStores;
    actionCreators = (0, _lodashObjectMapValues2['default'])(nextActionCreators, wrapActionCreator);

    // Merge the observers
    observers = (0, _lodashObjectMapValues2['default'])(stores, function (store, key) {
      return observers[key] || [];
    });

    // Dispatch to initialize stores
    if (currentTransaction) {
      updateState(committedState);
      currentTransaction.forEach(dispatch);
    } else {
      dispatch(BOOTSTRAP_STORE);
    }
  }

  // Support state transactions hooks for devtools.
  // Useful for hot-reloading some actions on top of a "committed" state.
  function transact() {
    if (currentTransaction) {
      throw new Error('Cannot nest transactions.');
    }

    currentTransaction = [];
    committedState = currentState;

    function finish(nextState) {
      currentTransaction = null;
      committedState = nextState;
      updateState(nextState);
    }

    function commit() {
      finish(currentState);
    }

    function rollback() {
      finish(committedState);
    }

    return { commit: commit, rollback: rollback };
  }

  return {
    getActions: getActions,
    observeStores: observeStores,
    receive: receive,
    transact: transact
  };
}

module.exports = exports['default'];

==> redux-0.2.2.tgz: package/lib/createDispatcher.js <==
'use strict';

var _Object$defineProperty = require('babel-runtime/core-js/object/define-property')['default'];

var _Object$keys = require('babel-runtime/core-js/object/keys')['default'];

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default')['default'];

_Object$defineProperty(exports, '__esModule', {
  value: true
});

exports['default'] = createDispatcher;

var _lodashObjectMapValues = require('lodash/object/mapValues');

var _lodashObjectMapValues2 = _interopRequireDefault(_lodashObjectMapValues);

var _invariant = require('invariant');

var _invariant2 = _interopRequireDefault(_invariant);

// An action dispatched to init store state
var BOOTSTRAP_STORE = {
  type: 'BOOTSTRAP_STORE'
};

function createDispatcher() {
  var observers = {};
  var stores = {};
  var actionCreators = {};
  var currentState = {};
  var currentTransaction = null;
  var committedState = {};

  // To compute the next state, combine the next states of every store
  function computeNextState(state, action) {
    return (0, _lodashObjectMapValues2['default'])(stores, function (store, key) {
      var nextStoreState = store(state[key], action);
      (0, _invariant2['default'])(nextStoreState != null, 'State returned by %s is null or undefined.', key);
      return nextStoreState;
    });
  }

  // Notify observers about the changed stores
  function emitChange(changedKeys) {
    if (!changedKeys.length) {
      return;
    }

    // Gather the affected observers
    var notifyObservers = [];
    changedKeys.forEach(function (key) {
      observers[key].forEach(function (o) {
        if (notifyObservers.indexOf(o) === -1) {
          notifyObservers.push(o);
        }
      });
    });

    // Emit change
    notifyObservers.forEach(function (o) {
      return o();
    });
  }

  // Update state and emit change if needed
  function updateState(nextState) {
    // Swap the state
    var previousState = currentState;
    currentState = nextState;

    // Notify the observers
    var changedKeys = _Object$keys(currentState).filter(function (key) {
      return currentState[key] !== previousState[key];
    });
    emitChange(changedKeys);
  }

  // Reassign the current state on each dispatch
  function dispatch(action) {
    (0, _invariant2['default'])(typeof action.type === 'string', 'Action type must be a string.');

    var nextState = computeNextState(currentState, action);
    updateState(nextState);
  }

  // Provide subscription and unsubscription
  function observeStores(observedKeys, onChange) {
    // Emit the state update
    function handleChange() {
      onChange(currentState);
    }

    // Synchronously emit the initial value
    handleChange();

    // Register the observer for each relevant key
    observedKeys.forEach(function (key) {
      return observers[key].push(handleChange);
    });

    // Let it unregister when the time comes
    return function () {
      observedKeys.forEach(function (key) {
        var index = observers[key].indexOf(handleChange);
        observers[key].splice(index, 1);
      });
    };
  }

  // Dispatch in the context of current transaction
  function dispatchInTransaction(action) {
    if (currentTransaction) {
      currentTransaction.push(action);
    }
    dispatch(action);
  }

  // Bind action creator to the dispatcher
  function wrapActionCreator(actionCreator) {
    return function dispatchAction() {
      for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }

      var action = actionCreator.apply(undefined, args);
      if (typeof action === 'function') {
        // Callback-style action creator
        action(dispatchInTransaction, currentState);
      } else {
        // Simple action creator
        dispatchInTransaction(action);
      }
    };
  }

  // Provide dispatching
  function getActions() {
    return actionCreators;
  }

  // Provide a way to receive new stores and actions
  function receive(nextStores, nextActionCreators) {
    stores = nextStores;
    actionCreators = (0, _lodashObjectMapValues2['default'])(nextActionCreators, wrapActionCreator);

    // Merge the observers
    observers = (0, _lodashObjectMapValues2['default'])(stores, function (store, key) {
      return observers[key] || [];
    });

    // Dispatch to initialize stores
    if (currentTransaction) {
      updateState(committedState);
      currentTransaction.forEach(dispatch);
    } else {
      dispatch(BOOTSTRAP_STORE);
    }
  }

  // Support state transactions hooks for devtools.
  // Useful for hot-reloading some actions on top of a "committed" state.
  function transact() {
    if (currentTransaction) {
      throw new Error('Cannot nest transactions.');
    }

    currentTransaction = [];
    committedState = currentState;

    function finish(nextState) {
      currentTransaction = null;
      committedState = nextState;
      updateState(nextState);
    }

    function commit() {
      finish(currentState);
    }

    function rollback() {
      finish(committedState);
    }

    return { commit: commit, rollback: rollback };
  }

  return {
    getActions: getActions,
    observeStores: observeStores,
    receive: receive,
    transact: transact
  };
}

module.exports = exports['default'];
